#!/usr/bin/python

#--------------------------- CONFIGURATION -----------------------

# Where is the java binary?
JAVA_PATH = "/usr/local/bin/java"

# Where is the file Multivalent.jar?
MULTIVALENT_PATH = "/usr/local/pkgs/Multivalent/Multivalent.jar"

#-------------------------- END CONFIGURATION --------------------


import commands
import sys
import tempfile
import string
import time
import os
import thread

# We use this to make sure that the conversion thread completed
# before we move on
convertLock = thread.allocate_lock()

# For the handful of spreadsheet manipulations we need to do
import gnumeric

from wxPython.wx import *

ID_TIMER = wxNewId()

# MAke sure we've got access to those configurations
try:
    assert(os.access(JAVA_PATH, os.R_OK|os.X_OK))
    assert(os.access(MULTIVALENT_PATH, os.R_OK))
except:
    sys.exit("Configuration failed!  Couldn't find one of java, Multivalent.")

#---------------------------------------------------------------------------

def doConvert(cmdLine, lock):
    '''This is really only used to do the command line conversion of the PDF
    into ppm.  Acquires the lock, runs the command, releases the lock.'''
    lock.acquire()
    commands.getstatusoutput(cmdLine)
    lock.release()

def curryComparisonGenerator(ind):
    return lambda x, y: (x[ind] < y[ind]) * -1 + (x[ind] > y[ind])

sortByUsername = curryComparisonGenerator(0)
sortByLast = curryComparisonGenerator(1)
sortByFirst = curryComparisonGenerator(2)
sortBySid = curryComparisonGenerator(3)

class TestFrame(wxFrame):
    def __init__(self, parent, log):
        # We are a frame, initialize as such
        wxFrame.__init__(self, parent, -1,
                         "PDF Splitter/Renamer", size=(800, 600))

        # If we don't have any arguments, find the file we want to work on
        if len(sys.argv) == 1:
            dlg = wxFileDialog(self, "Select PDF to split.",
                               style=wxOPEN)
            result = dlg.ShowModal()
            file = dlg.GetFilename()
            dlg.Destroy()
            if result != wxID_OK:
                self.Close()
                return
            sys.argv.append(file)

        # If we can't access that file, give up
        if not os.access(sys.argv[1], os.R_OK):
            dlg = wxMessageDialog(self,
                                  "Cannot open PDF for reading.",
                                  "Error",
                                  style=wxICON_ERROR|wxOK)
            dlg.ShowModal()
            dlg.Destroy()
            sys.exit(1)            

        # If our file is not a pdf, error
        if string.lower(sys.argv[1][-4:]) != ".pdf":
            dlg = wxMessageDialog(self,
                                  "Not a PDF file",
                                  "Error",
                                  style=wxICON_ERROR|wxOK)
            dlg.ShowModal()
            dlg.Destroy()
            sys.exit(0)

        # If we can find the gradebook based on where in /courses/ we were
        # invoked, try that
        if os.path.realpath(sys.argv[1])[:8] != "/courses":
            # If we weren't invoked in courses, then just try to find
            # the thing manually.
            dlg = wxFileDialog(self,
                               "Select gradebook file",
                               style=wxOPEN)
            response = dlg.ShowModal()
            rosterFn = dlg.GetPath()
            dlg.Destroy()

            # If they hit cancel, give up
            if response != wxID_OK:
                sys.exit(0)

        # Otherwise we are in /courses
        else:
            # Split path pieces
            tokens = os.path.realpath(sys.argv[1]).split("/")

            # The course number is /courses/csXXX[x]/ (note the leading /)
            courseNum = tokens[2][2:]

            # And thus this is where the roster ought to be
            rosterFn = "/courses/%s/%s/gradebook/%s.xls" % \
                       (tokens[2], tokens[3], courseNum)

            # If we didn't find it
            if not os.access(rosterFn, os.R_OK):
                # Locate it yourself if it ain't there
                dlg = wxFileDialog(self,
                                   "Select gradebook file",
                                   style=wxOPEN)
                response = dlg.ShowModal()
                rosterFn = dlg.GetPath()
                dlg.Destroy()

                # Again, if they hit cancel
                if response != wxID_OK:
                    sys.exit(0)

        # if we can't open it or it isn't there
        if not os.access(rosterFn, os.R_OK):
            dlg = wxMessageDialog(self,
                                  "Can't open roster for reading, aborting.",
                                  "Error",
                                  style=wxICON_ERROR|wxOK)
            dlg.ShowModal()
            dlg.Destroy()
            sys.exit(0)

        # The back panel
        self.p = p = wxPanel(self, -1)
        p.SetSize((920, 820))
        p.SetAutoLayout(True)

        # Build the widgets
        # This is the list of usernames etc.
        self.userList = wxListCtrl(p, -1, size=(440, 400),
                                   style=wxLC_HRULES|wxLC_VRULES|wxLC_REPORT)
        self.userList.InsertColumn(0, "Username")
        self.userList.InsertColumn(1, "Last")
        self.userList.InsertColumn(2, "First")
        self.userList.InsertColumn(3, "SID")

        # This is the list of assigned split points
        self.splitList = wxListCtrl(p, -1, size=(440, 200),
                                    style=wxLC_HRULES|wxLC_VRULES|wxLC_REPORT)
        self.splitList.InsertColumn(0, "Username")
        self.splitList.InsertColumn(1, "First Page")
        self.splitList.InsertColumn(2, "Last Page")
        

        # If the roster needs converting
        if rosterFn[-9:] != ".gnumeric":
            # Note the old name
            oldRoster = rosterFn

            # Get a new name
            rosterFn = tempfile.mkstemp(suffix=".gnumeric")[1]

            # mkstemp creates it on disk, and we don't want
            # ssconvert to complain at us
            os.unlink(rosterFn)
            converted = True

            # Convert it
            cmdLine = "ssconvert %s %s" % (oldRoster, rosterFn)
            status, output = commands.getstatusoutput(cmdLine)
            if status:
                print >>sys.stderr, output
                sys.exit(1)                
        else:
            converted = False

        # Get the sheet names so we can find the index of Summary
        sheetNames = gnumeric.sheetNames(rosterFn)
        try:
            gnumeric.selectSheet(rosterFn, sheetNames.index("Summary"))
        except ValueError:
            dlg = wxMessageDialog(self,
                                  "Roster file has no sheet named 'Summary',"+\
                                  " cannot extract usernames.  Aborting.",
                                  "Error",
                                  style=wxICON_ERROR|wxOK)
            dlg.ShowModal()
            dlg.Destroy()

            # If we converted it, delete it from /tmp
            if converted:
                os.unlink(rosterFn)
            sys.exit(0)        

        # Get a csv of it so we don't have indirect references
        csv = tempfile.mkstemp(suffix=".csv")[1]
        commands.getstatusoutput("ssconvert %s %s" % (rosterFn, csv))

        # Get the data from the roster
        csvDataRaw = open(csv).readlines()
        if converted:
            os.unlink(rosterFn)
        os.unlink(csv)

        # Read in the headers
        headers = csvDataRaw.pop(0).split(",")
        headers = map(string.lower, headers)

        # Try to find the headers for "usernane", "last", "first", and "sid",
        # if they don't exist, then just do as good as we can
        # (we've got defaults already)
        hmap = {"username": 2, "last": 0, "first": 1, "sid": 3}            
        try:
            hmap["username"] = headers.index("username")
        except:
            dlg = wxMessageDialog(self,
                                  "Unable to extract proper column " + \
                                  "information, using default columns of\n" +\
                                  "Summary.  May not be useful, or column " +\
                                  "headers may be wrong.",
                                  "Warning",
                                  style=wxICON_WARNING|wxOK)
            dlg.ShowModal()
            dlg.Destroy()
        try:
            hmap["first"] = headers.index("first")
        except:
            pass
        try:
            hmap["last"] = headers.index("last")
        except:
            pass
        try:
            hmap["sid"] = headers.index("sid")
        except:
            pass

        # Now read the data, twiddling the rows in the order dictated above
        data = []
        for line in csvDataRaw:
            tokens = line.split(",")
            try:
                cur = [tokens[hmap["username"]], tokens[hmap["last"]],
                            tokens[hmap["first"]], tokens[hmap["sid"]]]
            except:
                cur = tokens[:4]
                   
            data.append(cur)
        
        self.data = data

        # This is so we can always look things up by username (useful
        # for the "unset" button
        self.dataLookup = {}
        for username, last, first, sid in self.data:
            self.dataLookup[username] = (last, first, sid)
        
        # The main PDF view window
        self.bitmapPanel = wxScrolledWindow(p, -1)
        self.bitmapPanel.SetScrollbars(10, 10, 60, 80)
        self.bitmapPanel.SetSize((600, 800))

        # All of our buttons, with their tooltips
        self.nextPageBtn = wxButton(p, -1, "Next Page")        
        self.prevPageBtn = wxButton(p, -1, "Prev Page")
        self.setStartBtn = wxButton(p, -1, "Set Sub Start")
        self.setStartBtn.SetToolTip(wxToolTip(\
            "Set the beginning of the submission for the current " + \
            "student to this page."))
        
        self.addUserBtn = wxButton(p, -1, "Add Username")
        self.addUserBtn.SetToolTip(wxToolTip("Add a new username to " + \
                                             "the list."))
        
        self.undoBtn = wxButton(p, -1, "Unset break")
        self.undoBtn.Disable()
        self.undoBtn.SetToolTip(wxToolTip("Select a break from the lower "+\
                                          "list and click here to remove it."))
        
        self.doneBtn = wxButton(p, -1, "Done")
        self.doneBtn.SetToolTip(wxToolTip("Accept current breaks, split " + \
                                          "& rename."))

        ######################## EVENT HANDLERS ########################
        # This is the redraw for the bitmap, since it isn't a normal bitmap
        # (Actually, I realize that I just copied most of this out of
        # Agar's AnnotateFrame, but that isn't necessary: a wxStaticBitmap
        # with scroll bars would do the same.  I wonder if it is worth
        # fixing . . . . )
        EVT_PAINT(self.bitmapPanel, self.onPaint)
        EVT_SIZE(self, self.onSize)

        # Sort by column
        EVT_LIST_COL_CLICK(self.userList, -1, self.onColClick)

        # Button handlers
        EVT_BUTTON(self.nextPageBtn, -1, self.onNext)
        EVT_BUTTON(self.prevPageBtn, -1, self.onPrev)
        EVT_BUTTON(self.setStartBtn, -1, self.onSet)
        EVT_BUTTON(self.addUserBtn, -1, self.onAdd)
        EVT_BUTTON(self.undoBtn, -1, self.onUndo)
        EVT_BUTTON(self.doneBtn, -1, self.onDone)

        # Key press.  Frame doesn't process events, so we register
        # with most of the sizable widgets.
        EVT_KEY_DOWN(self.bitmapPanel, self.onKeyDown)
        EVT_KEY_DOWN(self.userList, self.onKeyDown)
        EVT_KEY_DOWN(self.splitList, self.onKeyDown)
        EVT_KEY_DOWN(p, self.onKeyDown)

        # Timer handler for the initial conversion
        EVT_TIMER(self, ID_TIMER, self.onConvert)

        # For double-clicking on usernames and such
        EVT_LIST_ITEM_ACTIVATED(self, -1, self.onSet)


        # Can't go back, to start
        self.prevPageBtn.Disable()

        # Pack it in
        self.sizer = wxBoxSizer(wxVERTICAL)
        bs = wxBoxSizer(wxHORIZONTAL)
        bs.Add(self.bitmapPanel, 0, wxEXPAND|wxGROW|wxALL, 5)

        vs = wxBoxSizer(wxVERTICAL)
        vs.Add(self.userList, 0, wxEXPAND|wxGROW|wxALL)
        vs.Add(self.splitList, 0, wxEXPAND|wxGROW|wxALL)
        
        bs.Add(vs, 0, wxEXPAND|wxGROW|wxALL)
        self.sizer.Add(bs)

        buttonSizer = wxBoxSizer(wxHORIZONTAL)
        buttonSizer.Add(self.prevPageBtn)
        buttonSizer.Add(self.nextPageBtn)
        buttonSizer.Add(self.setStartBtn)
        buttonSizer.Add(self.addUserBtn)
        buttonSizer.Add(self.undoBtn)
        buttonSizer.Add(self.doneBtn)

        self.sizer.Add(buttonSizer)
        
        p.SetSizer(self.sizer)
        self.sizer.Layout()
        self.sizer.Fit(self)

        # We start the conversion after .5 seconds.  I suppose this
        # could be re-organized now that I've broken this out into
        # its own thread . . . not worth it really.
        self.convertTimer = wxTimer(self, ID_TIMER)
        self.convertTimer.Start(500, True)

        # Draw the userlist based on the data in self.data
        self.redrawUserList()

    def onKeyDown(self, event):
        # Right cursor
        if event.GetKeyCode() == 318:
            self.onNext(None)
        # Left cursor
        elif event.GetKeyCode() == 316:
            self.onPrev(None)
        # Escape
        elif event.GetKeyCode() == 27:
            dlg = wxMessageDialog(self,
                                  "Really quit?",
                                  "Confirm",
                                  style=wxYES_NO|wxICON_QUESTION)
            response = dlg.ShowModal()
            dlg.Destroy()
            if response == wxID_YES:
                self.Destroy()

    def redrawUserList(self):
        # Empty it out, which clears column info too
        self.userList.ClearAll()

        # Re-add the columns
        self.userList.InsertColumn(0, "Username")
        self.userList.InsertColumn(1, "Last")
        self.userList.InsertColumn(2, "First")
        self.userList.InsertColumn(3, "SID")

        # This is reall just len(self.data)
        self.numItems = 0

        # Loop through and add everything
        for ind, row in enumerate(self.data):
            try:
                username, last, first, sid = row[:4]
            except:
                continue
            self.userList.InsertStringItem(ind, username)
            self.numItems += 1

            self.userList.SetStringItem(ind, 1, last)
            self.userList.SetStringItem(ind, 2, first)
            self.userList.SetStringItem(ind, 3, sid)


    def redrawSplitList(self):
        # Like redrawUserList, but for the split list (self.pageBreaks)
        self.splitList.ClearAll()
        self.splitList.InsertColumn(0, "Username")
        self.splitList.InsertColumn(1, "First Page")
        self.splitList.InsertColumn(2, "Last Page")

        for i in range(len(self.pageBreaks) - 1):
            self.splitList.InsertStringItem(i, self.pageBreaks[i][1])
            self.splitList.SetStringItem(i, 1, str(self.pageBreaks[i][0]))
            self.splitList.SetStringItem(i, 2,
                                         str(self.pageBreaks[i + 1][0] - 1))

        # Note that the last one is special, since it goes to the end.
        if self.pageBreaks:
            i = len(self.pageBreaks) - 1
            self.splitList.InsertStringItem(i,
                                            self.pageBreaks[-1][1])
            self.splitList.SetStringItem(i, 1, str(self.pageBreaks[-1][0]))
            self.splitList.SetStringItem(i, 2, str(self.numpages))
            
        
        
    def onColClick(self, event):
        # Get the name of the column that was clicked
        try:
            colname = self.userList.GetColumn(event.GetColumn()).GetText()
        except:
            return

        # Get the comparison functions that are to be used on rows
        # of self.data
        if colname == "Username":
            sortFunc = sortByUsername
        elif colname == "Last":
            sortFunc = sortByLast
        elif colname == "First":            
            sortFunc = sortByFirst
        else:
            sortFunc = sortBySid

        # Sort it
        self.data.sort(sortFunc)

        # And update the GUI
        self.redrawUserList()

    def onUndo(self, event):
        # Figure out which split we are undoing
        selInd = self.splitList.GetNextItem(-1,
                                            wxLIST_NEXT_ALL,
                                            wxLIST_STATE_SELECTED)
        if selInd == -1:
            dlg = wxMessageDialog(self,
                                  "Must select break from lower list",
                                  "Error",
                                  style=wxICON_ERROR|wxOK)
            dlg.ShowModal()
            dlg.Destroy()
            return

        # Get the username
        username = str(self.splitList.GetItemText(selInd))

        # Get the old data
        last, first, sid = self.dataLookup[username]

        # Get rid of the split
        self.pageBreaks.pop(selInd)

        # Re-add the data
        self.data.append([username, last, first, sid])

        # Redraw everything
        self.redrawUserList()
        self.redrawSplitList()

        # Disable the undo if there is nothing left to undo
        if not self.pageBreaks:
            self.undoBtn.Disable()

    def onConvert(self, event):
        # A nice little indication that we are busy
        cursor = wxBusyCursor();

        # How many pages are we doing?        
        cmdLine = "%s -classpath %s tool.pdf.Info %s" % \
                  (JAVA_PATH, MULTIVALENT_PATH, sys.argv[1])
        status, output = commands.getstatusoutput(cmdLine)
        if not status:
            output = output.split("\n")
            for line in output:
                if line[:10] == "Page count":
                    numpages = int(line.split(" ")[2])

        else:
            dlg = wxMessageDialog(self,
                                  "Error in getting PDF info, aborting.",
                                  "ERROR",
                                  wxICON_ERROR|wxOK)
            dlg.ShowModal()
            dlg.Destroy()
            sys.exit(1)            

        # Make a temp dir to hold our .ppm files
        self.dirname = dirname = tempfile.mkdtemp()

        # get rid of trailing /'s
        while self.dirname[-1] == '/':
            self.dirname = self.dirname[:-1]

        # Copy the file into temp
        commands.getoutput("cp %s %s/%s" % (sys.argv[1], dirname, sys.argv[1]))

        # Create the command line for doing the conversion
        cmdLine = os.path.dirname(sys.argv[0]) + "/convertPDF.sh %s %s" % \
                                      (dirname, dirname + "/" + sys.argv[1])

        # Start the thread that does the conversion.  It will grab
        # the lock
        global convertLock
        thread.start_new_thread(doConvert, (cmdLine, convertLock))

        # A progress meter
        progress = wxProgressDialog("Processing PDF",
                                    "0/%d pages" % numpages,
                                    maximum=numpages + 1)

        # Now we do a delayed busy-wait until all the files have been
        # created.
        # TODO: Might want something to make sure that convertPDF.sh is
        # still running and to error out if that fails.  We'll see if
        # that is necessary.
        done = False
        while not done:
            # Count the number of .ppms in that directory
            files = os.listdir(dirname)
            count = 0
            for file in files:
                if file[-4:] == ".ppm": count += 1

            # Oooh, got 'em all
            if count == numpages:
                done = True

            # Update the meter and go back to bed
            progress.Update(count, "%d/%d pages" % (count, numpages))
            time.sleep(.5)

        # Make sure we haven't managed the race-condition of the file
        # being touched but not yet filled with data, only important
        # for the single-page case (and why are we splitting that?)
        # This is really more for peace of mind than actual
        # safety, but I'm paranoid right now
        convertLock.acquire()
        convertLock.release()

        # Clear 'em up
        del convertLock        
        progress.Update(numpages + 1)
        progress.Destroy()
        
        del progress
        del cursor

        # Set state
        self.numpages = numpages
        self.curPage = 1

        self.pageBreaks = []

        # Load the first page
        try:
            assert(os.access(self.makeFilename(self.curPage), os.R_OK))
        except AssertionError:
            print os.listdir(self.dirname)
            raise
        
        self.bitmap = wxBitmap(self.makeFilename(self.curPage))
        self.redraw()

    def __del__(self):
        # Clean up our working directory
        commands.getstatusoutput("rm -rf " + self.dirname)

    def onAdd(self, event):
        # Need a new username?
        dlg = wxTextEntryDialog(self, "Enter the new username",
                                "Username?")
        response = dlg.ShowModal()
        text = dlg.GetValue()
        dlg.Destroy()

        if response != wxID_OK:
            return

        self.data.append([text, "", "", ""])
        self.numItems += 1
        self.redrawUserList()

    def onDone(self, event):
        # One last check before we do the splits
        message = "Break like this?\n\n"
        for i in range(1, len(self.pageBreaks)):
            message += "%s: %d-%d\n" % (self.pageBreaks[i - 1][1],
                                      self.pageBreaks[i - 1][0],
                                      self.pageBreaks[i][0] - 1)

        message += "%s: %d-%d\n" % (self.pageBreaks[-1][1],
                                    self.pageBreaks[-1][0],
                                    self.numpages)
        dlg = wxMessageDialog(self,
                              message,
                              "Confirm",
                              style=wxICON_QUESTION|wxYES_NO)
        response = dlg.ShowModal()
        dlg.Destroy()

        # Oooh, stop!
        if response != wxID_YES:
            return

        # Now we just do the conversions
        progress = wxProgressDialog("Splitting and renaming", "Splitting",
                                    maximum=len(self.pageBreaks))

        # Convert one split at a time
        for i in range(1, len(self.pageBreaks)):
            cmdLine = "%s -classpath %s tool.pdf.Split -page %d-%d %s" % \
                      (JAVA_PATH, MULTIVALENT_PATH,
                       self.pageBreaks[i - 1][0],
                       self.pageBreaks[i][0] - 1,
                       sys.argv[1])
            progress.Update(i - 1, "Converting " + self.pageBreaks[i - 1][1])
            (status, output) = commands.getstatusoutput(cmdLine)
            if status:
                print >>sys.stderr, output
                sys.exit(1)

            try:
                if os.access(self.pageBreaks[i - 1][1] + ".pdf", os.R_OK):
                    os.unlink(self.pageBreaks[i - 1][1] + ".pdf")
                    
                os.link(sys.argv[1][:-4] + "-x.pdf",
                        self.pageBreaks[i - 1][1] + ".pdf")
                os.unlink(sys.argv[1][:-4] + "-x.pdf")
            except OSError:
                print "Failed to move %s-x.pdf" % sys.argv[1][:-4]
                print "to %s" + self.pageBreaks[i - 1][1] + ".pdf"
                raise

        # The last one is special
        progress.Update(len(self.pageBreaks),
                            "Converting " + self.pageBreaks[-1][1])
        cmdLine = "%s -classpath %s tool.pdf.Split -page %d-%d %s" % \
                  (JAVA_PATH, MULTIVALENT_PATH,
                   self.pageBreaks[-1][0],
                   self.numpages,
                   sys.argv[1])
        (status, output) = commands.getstatusoutput(cmdLine)

        if os.access(self.pageBreaks[-1][1] + ".pdf", os.R_OK):
            os.unlink(self.pageBreaks[-1][1] + ".pdf")
            
        os.link(sys.argv[1][:-4] + "-x.pdf",
                self.pageBreaks[-1][1] + ".pdf")
        os.unlink(sys.argv[1][:-4] + "-x.pdf")

        if status:
            print >>sys.stderr, output
            sys.exit(1)

        progress.Destroy()

        # All done
        dlg = wxMessageDialog(self,
                              "PDF successfully split and renamed.",
                              "Converted.",
                              style=wxICON_INFORMATION|wxOK)
        dlg.ShowModal()
        dlg.Destroy()

        
        self.Destroy()
        

    def makeFilename(self, page):
        # Get the filename based on our working directory and the page
        return "%s/tmp-%d.ppm" % (self.dirname, page)

    def onPrev(self, event):
        # Can we go back a page?
        if self.curPage > 1:
            self.curPage -= 1
            self.bitmap = wxBitmap(self.makeFilename(self.curPage))
            self.redraw()

    def onNext(self, event):
        # Can we go forward a page?
        if self.curPage < self.numpages:
            self.curPage += 1
            self.bitmap = wxBitmap(self.makeFilename(self.curPage))
            self.redraw()

    def onSet(self, event):
        # Find which username is selected, if any
        selInd = self.userList.GetNextItem(-1,
                                           wxLIST_NEXT_ALL,
                                           wxLIST_STATE_SELECTED)
        if selInd == -1:
            return

        # Get the data
        username = str(self.userList.GetItemText(selInd))
        last = str(self.userList.GetItem(selInd, 1).GetText())
        first = str(self.userList.GetItem(selInd,2).GetText())
        sid = str(self.userList.GetItem(selInd, 3).GetText())

        # If this is the first break, then enable the unset button
        if not self.pageBreaks:
            self.undoBtn.Enable()

        # Note the new break
        self.pageBreaks.append([self.curPage, username, last, first, sid])
        self.pageBreaks.sort()

        # Update the gui
        self.redrawSplitList()

        # Remove the current username
        for ind, row in enumerate(self.data):
            if row[0] == username:
                del self.data[ind]
                break

        # Update the gui
        self.redrawUserList()

        # Go to the next page
        self.onNext(None)

    def redraw(self):
        # Manually redraw the PDF
        dc = wxClientDC(self.bitmapPanel)
        self.bitmapPanel.PrepareDC(dc)
        dc.Clear()
        self.draw(dc)

    def onSize(self, event):
        # Resize things manually because 2.5.x isn't doing it nicely for me
        # Not sure why.
        size = event.GetSize()
        if size[1] < 300: size[1] = 300
        self.p.SetSize(size)
        self.p.SetMinSize(size)
        x = int(event.GetSize()[0] / 2) - 10
        y = event.GetSize()[1] - 60
        if x > 100:
            delta = int(x * 0.25)
        else:
            delta = 0
            
        self.bitmapPanel.SetSize((x + delta, y))
        self.bitmapPanel.SetMinSize((x + delta, y))
        self.userList.SetSize((x - delta, int(y * 2./3)))
        self.userList.SetMinSize((x - delta, int(y * 2./3)))
        self.splitList.SetSize((x - delta, int(y * 1./3)))
        self.splitList.SetMinSize((x - delta, int(y * 1./3)))
        self.sizer.RecalcSizes()
        self.sizer.Layout()
        
    def onPaint(self, event):
        # This is the repain-event handler
        dc = wxPaintDC(self.bitmapPanel)
        self.bitmapPanel.PrepareDC(dc)
        dc.Clear()
        self.draw(dc)

    def draw(self, dc):
        # Draw the bitmap panel
        if hasattr(self, "bitmap"):
            x = int((600 - self.bitmap.GetWidth()) / 2)
            y = int((800 - self.bitmap.GetHeight()) / 2)
            
            dc.DrawBitmap(self.bitmap, x, y, False)

        dc.EndDrawing()

        if not hasattr(self, "curPage"):
            return

        # Make sure the correct buttons are disabled
        if self.curPage == 1:
            self.prevPageBtn.Disable()
        else:
            self.prevPageBtn.Enable()
            
        if self.curPage == self.numpages:
            self.nextPageBtn.Disable()
        else:
            self.nextPageBtn.Enable()

#---------------------------------------------------------------------------

if __name__ == '__main__':
    import sys
    
    app = wxPySimpleApp()
    frame = TestFrame(None, sys.stdout)
    frame.Show(true)
    app.MainLoop()

#---------------------------------------------------------------------------
